package com.bpcl.lpg.core.service.impl;

import org.slf4j.LoggerFactory;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.lang.StringUtils;
import java.util.Objects;
import com.fasterxml.jackson.core.JsonProcessingException;
import java.net.http.HttpResponse;
import com.bpcl.lpg.core.pojo.LpgContactUpdateResponseData;
import com.bpcl.lpg.facades.data.ContactUpdateRequestData;
import com.bpcl.lpg.core.service.BpclHttpClientService;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import org.slf4j.Logger;
import com.bpcl.lpg.core.service.LpgContactNumberUpdateService;

public class LpgContactNumberUpdateServiceImpl implements LpgContactNumberUpdateService
{
    private static final Logger LOG;
    private ConfigurationService configurationService;
    private BpclHttpClientService bpclHttpClientService;
    
    public LpgContactUpdateResponseData contactUpdate(final ContactUpdateRequestData requestData) throws JsonProcessingException {
        final String endPointName = requestData.getEndPointName();
        final String endPoint = this.getConfigurationService().getConfiguration().getString(endPointName);
        final String jsonBody = this.getRequestJson(requestData, endPoint);
        final HttpResponse<String> response = this.getBpclHttpClientService().executePostMethod(endPoint, jsonBody, endPointName);
        return this.populateLpgContactUpdateResponseData(response);
    }
    
    public LpgContactUpdateResponseData populateLpgContactUpdateResponseData(final HttpResponse<String> httpResponse) {
        if (Objects.nonNull(httpResponse) && StringUtils.isNotBlank((String)httpResponse.body())) {
            final String body = httpResponse.body();
            final ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            try {
                return mapper.readValue(body, LpgContactUpdateResponseData.class);
            }
            catch (final JsonProcessingException jsonProcessingException) {
                LpgContactNumberUpdateServiceImpl.LOG.error("Error while converting json to object", jsonProcessingException);
            }
        }
        return null;
    }
    
    private String getRequestJson(final ContactUpdateRequestData requestData, final String endPointName) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();
        final String json = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(requestData);
        return objectMapper.writeValueAsString(requestData);
    }
    
    public ConfigurationService getConfigurationService() {
        return this.configurationService;
    }
    
    public void setConfigurationService(final ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }
    
    public BpclHttpClientService getBpclHttpClientService() {
        return this.bpclHttpClientService;
    }
    
    public void setBpclHttpClientService(final BpclHttpClientService bpclHttpClientService) {
        this.bpclHttpClientService = bpclHttpClientService;
    }
    
    static {
        LOG = LoggerFactory.getLogger(LpgContactNumberUpdateServiceImpl.class);
    }
}
